"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * コールバックコントローラーの抽象クラス
 * 以下のメソッドをオーバーライドして使用する
 * * [[init]] (定義しなくてもよい)
 * * [[callLex]]
 * * [[callGrammar]]
 */
class AbstractCallbackController {
    constructor(_language) {
        this._language = _language;
    }
    get language() {
        return this._language;
    }
    /**
     * 解析を開始する際、初期化のために呼び出される
     * デフォルトでは何もしない
     */
    init() {
        /* do nothing */
    }
}
exports.AbstractCallbackController = AbstractCallbackController;
/**
 * 言語情報に付随するコールバックを呼び出すコントローラ
 */
class DefaultCallbackController extends AbstractCallbackController {
    callLex(id, value, lexer) {
        const rule = this.language.lex.rules[id];
        if (rule.callback !== undefined) {
            return rule.callback(value, rule.token, lexer);
        }
        else {
            return value;
        }
    }
    callGrammar(id, children, lexer) {
        const rule = this.language.grammar.rules[id];
        if (rule.callback !== undefined) {
            return rule.callback(children, rule.ltoken, lexer);
        }
        else {
            return children[0];
        }
    }
}
exports.DefaultCallbackController = DefaultCallbackController;
/**
 * ASTを構築するコントローラ
 */
class ASTConstructor extends AbstractCallbackController {
    callLex(id, value, lexer) {
        return {
            type: this.language.lex.rules[id].token,
            value,
            children: []
        };
    }
    callGrammar(id, children, lexer) {
        return {
            type: this.language.grammar.rules[id].ltoken,
            value: null,
            children: children
        };
    }
}
exports.ASTConstructor = ASTConstructor;
