import { Token } from "../def/token";
import { ClosureSet } from "./closureset";
import { GrammarDB } from "./grammardb";
export declare type DFAEdge = Map<Token, number>;
export declare type DFANode = {
    closure: ClosureSet;
    edge: DFAEdge;
};
export declare type DFA = Array<DFANode>;
/**
 * 構文規則からLR(1)DFAおよびLALR(1)DFAを生成する
 */
export declare class DFAGenerator {
    private grammardb;
    private lr_dfa;
    private lalr_dfa;
    /**
     * @param {GrammarDB} grammardb 構文規則
     */
    constructor(grammardb: GrammarDB);
    /**
     * LR(1)DFAを得る
     * @returns {DFA}
     */
    getLR1DFA(): DFA;
    /**
     * LALR(1)DFAを得る
     * @returns {DFA}
     */
    getLALR1DFA(): DFA;
    /**
     * DFAの生成
     */
    private generateDFA();
    /**
     * LR(1)オートマトンの先読み部分をマージして、LALR(1)オートマトンを作る
     */
    private mergeLA();
    /**
     * 既存のClosureSetから新しい規則を生成し、対応する記号ごとにまとめる
     * @param closureset
     */
    private generateNewClosureSets(closureset);
}
