import { GrammarDefinition } from "../def/language";
import { Token } from "../def/token";
/**
 * 終端/非終端記号の判別を行う
 */
export declare class SymbolDiscriminator {
    private terminal_symbols;
    private nonterminal_symbols;
    constructor(grammar: GrammarDefinition);
    /**
     * 終端記号の集合をSetで得る
     * @param {boolean} prevent_copy trueを与えるとSetをコピーせず返す
     * 結果に変更が加えられないと保証される場合に用いる
     * @returns {Set<Token>}
     */
    getTerminalSymbols(prevent_copy?: boolean): Set<Token>;
    /**
     * 非終端記号の集合をSetで得る
     * @param {boolean} prevent_copy trueを与えるとSetをコピーせず返す
     * 結果に変更が加えられないと保証される場合に用いる
     * @returns {Set<Token>}
     */
    getNonterminalSymbols(prevent_copy?: boolean): Set<Token>;
    /**
     * 与えられた記号が終端記号かどうかを調べる
     * @param {Token} symbol
     * @returns {boolean}
     */
    isTerminalSymbol(symbol: Token): boolean;
    /**
     * 与えられた記号が非終端記号かどうかを調べる
     * @param {Token} symbol
     * @returns {boolean}
     */
    isNonterminalSymbol(symbol: Token): boolean;
}
