"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const token_1 = require("../def/token");
const parsergenerator_1 = require("../parsergenerator/parsergenerator");
const ruleparser_1 = require("./ruleparser");
/**
 * 予め構文解析器を生成しておいて利用するためのソースコードを生成する
 */
class PreCompiler {
    /**
     * @param import_path パーサジェネレータをimportするためのディレクトリパス
     */
    constructor(import_path = "lavriapg") {
        this.import_path = import_path;
        if (import_path[import_path.length - 1] != "/")
            this.import_path += "/";
    }
    /**
     * 構文ファイルを受け取り、それを処理できるパーサを構築するためのソースコードを返す
     * @param {string} input 言語定義文法によって記述された、解析対象となる言語
     * @returns {string} 生成されたパーサのソースコード
     */
    exec(input) {
        const language = ruleparser_1.language_parser.parse(input);
        const parsing_table = new parsergenerator_1.ParserGenerator(language).getParsingTable();
        let result = "";
        result += `import {Token, SYMBOL_EOF} from "${this.import_path}def/token";
import {Language} from "${this.import_path}def/language";
import {ParsingOperation, ParsingTable} from "${this.import_path}def/parsingtable";
import {Parser} from "${this.import_path}parser/parser";
import {ParserFactory} from "${this.import_path}parser/factory";

export const language: Language = {
	lex: {
		rules: [
${language.lex.rules.map(({ token, pattern }) => {
            return `\t\t\t{token: ${token === null ? "null" : `"${token}"`}, ` +
                `pattern: ${pattern instanceof RegExp ? pattern : `"${pattern}"`}}`;
        }).join(",\n")}
		]
	},
	grammar: {
		rules: [
${language.grammar.rules.map(({ ltoken, pattern }) => `\t\t\t{
				ltoken: "${ltoken}",
				pattern: [${pattern.map((t) => `"${t}"`).join(", ")}]
			}`).join(",\n")}
		],
		start_symbol: "${language.grammar.start_symbol}"
	}
};

export const parsing_table: ParsingTable = [
${parsing_table.map((row) => `\tnew Map<Token, ParsingOperation>([
${(() => {
            let line = "";
            for (const [key, value] of row) {
                line += `\t\t[${key === token_1.SYMBOL_EOF ? "SYMBOL_EOF" : `"${key}"`}, ${JSON.stringify(value)}],\n`;
            }
            return line.slice(0, -2);
        })()}`).join("\n\t]),\n")}
	])
];

export const parser: Parser = ParserFactory.create(language, parsing_table);
`;
        return result;
    }
}
exports.PreCompiler = PreCompiler;
