"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const parsergenerator_1 = require("./parsergenerator/parsergenerator");
const factory_1 = require("./parser/factory");
const input = fs_1.readFileSync("/dev/stdin", "utf8");
const grammar = {
    rules: [
        {
            ltoken: "EXP",
            pattern: ["EXP", "PLUS", "TERM"],
            callback: (c) => c[0] + c[2]
        },
        {
            ltoken: "EXP",
            pattern: ["TERM"]
        },
        {
            ltoken: "TERM",
            pattern: ["TERM", "ASTERISK", "ATOM"],
            callback: (c) => c[0] * c[2]
        },
        {
            ltoken: "TERM",
            pattern: ["ATOM"]
        },
        {
            ltoken: "ATOM",
            pattern: ["DIGITS"],
            callback: (c) => +c[0]
        },
        {
            ltoken: "ATOM",
            pattern: ["LPAREN", "EXP", "RPAREN"],
            callback: (c) => c[1]
        }
    ],
    start_symbol: "EXP"
};
const lex = {
    rules: [
        { token: "DIGITS", pattern: /[1-9][0-9]*/ },
        { token: "PLUS", pattern: "+" },
        { token: "ASTERISK", pattern: "*" },
        { token: "LPAREN", pattern: "(" },
        { token: "RPAREN", pattern: ")" },
        { token: null, pattern: /(\r\n|\r|\n)+/ },
        { token: null, pattern: /[ \f\t\v\u00a0\u1680\u180e\u2000-\u200a\u202f\u205f\u3000\ufeff]+/ },
        { token: "INVALID", pattern: /./ }
    ]
};
const language = {
    lex: lex,
    grammar: grammar
};
console.time("process");
console.log(JSON.stringify(factory_1.ParserFactory.create(language, new parsergenerator_1.ParserGenerator(language).getParsingTable()).parse(input), undefined, 2));
console.timeEnd("process");
